package com.sun.org.apache.bcel.internal.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class LocalVariable implements Cloneable, Node
{
    private int start_pc;
    private int length;
    private int name_index;
    private int signature_index;
    private int index;
    private ConstantPool constant_pool;
    private int orig_index;
    
    public LocalVariable(LocalVariable c) {
        this(c.getStartPC(), c.getLength(), c.getNameIndex(), c.getSignatureIndex(), c.getIndex(), c.getConstantPool());
        this.orig_index = c.getOrigIndex();
    }
    
    LocalVariable(DataInput file, ConstantPool constant_pool) throws IOException {
        this(file.readUnsignedShort(), file.readUnsignedShort(), file.readUnsignedShort(), file.readUnsignedShort(), file.readUnsignedShort(), constant_pool);
    }
    
    public LocalVariable(int start_pc, int length, int name_index, int signature_index, int index, ConstantPool constant_pool) {
        this.start_pc = start_pc;
        this.length = length;
        this.name_index = name_index;
        this.signature_index = signature_index;
        this.index = index;
        this.constant_pool = constant_pool;
        this.orig_index = index;
    }
    
    public LocalVariable(int start_pc, int length, int name_index, int signature_index, int index, ConstantPool constant_pool, int orig_index) {
        this.start_pc = start_pc;
        this.length = length;
        this.name_index = name_index;
        this.signature_index = signature_index;
        this.index = index;
        this.constant_pool = constant_pool;
        this.orig_index = orig_index;
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitLocalVariable(this);
    }
    
    public final void dump(DataOutputStream file) throws IOException {
        file.writeShort(this.start_pc);
        file.writeShort(this.length);
        file.writeShort(this.name_index);
        file.writeShort(this.signature_index);
        file.writeShort(this.index);
    }
    
    public final ConstantPool getConstantPool() {
        return this.constant_pool;
    }
    
    public final int getLength() {
        return this.length;
    }
    
    public final String getName() {
        ConstantUtf8 c = (ConstantUtf8)this.constant_pool.getConstant(this.name_index, (byte)1);
        return c.getBytes();
    }
    
    public final int getNameIndex() {
        return this.name_index;
    }
    
    public final String getSignature() {
        ConstantUtf8 c = (ConstantUtf8)this.constant_pool.getConstant(this.signature_index, (byte)1);
        return c.getBytes();
    }
    
    public final int getSignatureIndex() {
        return this.signature_index;
    }
    
    public final int getIndex() {
        return this.index;
    }
    
    public final int getOrigIndex() {
        return this.orig_index;
    }
    
    public final int getStartPC() {
        return this.start_pc;
    }
    
    final String toStringShared(boolean typeTable) {
        String name = this.getName();
        String signature = Utility.signatureToString(this.getSignature(), false);
        String label = "LocalVariable" + (typeTable ? "Types" : "");
        return label + "(start_pc = " + this.start_pc + ", length = " + this.length + ", index = " + this.index + ":" + signature + " " + name + ")";
    }
    
    public final void setConstantPool(ConstantPool constant_pool) {
        this.constant_pool = constant_pool;
    }
    
    public final void setLength(int length) {
        this.length = length;
    }
    
    public final void setNameIndex(int name_index) {
        this.name_index = name_index;
    }
    
    public final void setSignatureIndex(int signature_index) {
        this.signature_index = signature_index;
    }
    
    public final void setIndex(int index) {
        this.index = index;
    }
    
    public final void setStartPC(int start_pc) {
        this.start_pc = start_pc;
    }
    
    @Override
    public final String toString() {
        return this.toStringShared(false);
    }
    
    public LocalVariable copy() {
        try {
            return (LocalVariable)this.clone();
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
    }
}
